@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-# $Id: Getopt.pm,v 1.28 1999/06/09 21:58:23 eserte Exp $
+# $Id: Getopt.pm,v 1.31 1999/08/31 10:03:20 eserte Exp $
 # Author: Slaven Rezic
 #
 # Copyright (C) 1997,1998,1999 Slaven Rezic. All rights reserved.
@@ -19,7 +19,7 @@ use vars qw($loadoptions $VERSION $x11_pass_through
 	    $CHECKMARK_OFF $CHECKMARK_ON $DEBUG
 	   );
 
-$VERSION = '0.36';
+$VERSION = '0.38';
 
 $DEBUG = 0;
 $x11_pass_through = 0;
@@ -101,7 +101,7 @@ sub new {
 		    };
 		    $varref = \$self->{'options'}{$o};
 		}
-		if (ref $varref eq 'SCALAR') {
+		if (ref($varref) =~ /^(SCALAR|HASH|ARRAY)$/) {
 		    $a{'var'} = $varref;
 		} else {
 		    die "Can't handle variable reference of type "
@@ -187,6 +187,12 @@ sub load_options {
 	if (exists $loadoptions->{$opt->[0]}) {
 	    if (ref $self->_varref($opt) eq 'CODE') {
 		&{$self->_varref($opt)} if $loadoptions->{$opt->[0]};
+	    } elsif (ref $self->_varref($opt) eq 'ARRAY' &&
+		     ref $loadoptions->{$opt->[0]} eq 'ARRAY') {
+		@{ $self->_varref($opt) } = @{ $loadoptions->{$opt->[0]} };
+	    } elsif (ref $self->_varref($opt) eq 'HASH' &&
+		     ref $loadoptions->{$opt->[0]} eq 'HASH') {
+		%{ $self->_varref($opt) } = %{ $loadoptions->{$opt->[0]} };
 	    } else {
 		$ {$self->_varref($opt)} = $loadoptions->{$opt->[0]};
 	    }
@@ -208,8 +214,13 @@ sub save_options {
 	    my %saveoptions;
 	    my $opt;
 	    foreach $opt ($self->_opt_array) {
-		$saveoptions{$opt->[0]} = $ {$self->_varref($opt)}
-		  if !$opt->[3]{'nosave'} && ref $self->_varref($opt) eq 'SCALAR';
+		if (!$opt->[3]{'nosave'}) {
+		    if (ref($self->_varref($opt)) eq 'SCALAR') {
+			$saveoptions{$opt->[0]} = $ {$self->_varref($opt)}
+		    } elsif (ref($self->_varref($opt)) =~ /^(HASH|ARRAY)$/) {
+			$saveoptions{$opt->[0]} = $self->_varref($opt);
+		    } 
+		}
 	    }
 	    if (Data::Dumper->can('Dumpxs')) {
 		# use faster version of Dump
@@ -486,7 +497,11 @@ sub _filedialog_widget {
 		   $file = $filedialog->Show(-directory => $dir);
 	       }
 	   } else {
-	       $file = $filedialog->Show;
+	       if ($fd eq 'getOpenFile') {
+		   $file = $topframe->getOpenFile(-title => 'Select file');
+	       } else {
+		   $file = $filedialog->Show;
+	       }
 	   }
 	   if ($file) {
 	       $ {$self->_varref($opt)} = $file;
@@ -638,7 +653,6 @@ sub option_editor {
 					  ? (-statusbar => $statusbar)
 					  : ());
     }
-    $opt_notebook->pack(-expand => 1, -fill => 'both');
 
     my $optlist = {};
     my $current_top;
@@ -677,10 +691,6 @@ sub option_editor {
 	}
     }
 
-    if (defined $statusbar) {
-	$statusbar->pack(-fill => 'x', -anchor => 'w');
-    }
-
     require Tk::Tiler;
     my $f;
     $f = $opt_editor->Tiler
@@ -698,10 +708,12 @@ sub option_editor {
 	   }
 	   $f->GeometryRequest($f->Width,
 			       2*$bw+$rows*$f->{Sh});
-       })->pack(-fill => 'x');
+       });
     $f->bind('<Configure>' => sub {
-		 if ($f->y + $f->height > $top->height) {
-		     $top->geometry($top->width."x".($f->height+$f->y));
+		 if ($f->y + $f->height > $opt_editor->height) {
+		     $opt_editor->geometry($opt_editor->width .
+					   "x" .
+					   ($f->height+$f->y));
 		 }
 	     });
     my @tiler_b;
@@ -783,6 +795,12 @@ sub option_editor {
 
     $opt_editor->bind('<Escape>' => sub { $cancel_button->invoke });
 
+    $f->pack(-fill => 'x', -side => "bottom");
+    $opt_notebook->pack(-expand => 1, -fill => 'both');
+    if (defined $statusbar) {
+	$statusbar->pack(-fill => 'x', -anchor => 'w');
+    }
+
     if ($opt_editor->can('Popup')) {
 	$opt_editor->Popup;
     }
